/*
  thorfinn thorfinn@binf.ku.dk dec17 2012
 
    
  anders albrecht@binf.ku.dk made this.

  part of angsd
*/


#include "shared.h"
#include <cmath>
#include "analysisFunction.h"
#include<cstdlib>

class ansErr:public general{
public:
  int doAnsError;
  int nInd;
  int sample;
  //none optional stuff
  FILE *outfile;
  FILE *outfile2;
  int currentChr;
  ansErr(const char *outfiles,argStruct *arguments,int inputtype);
  ~ansErr();
  void getOptions(argStruct *arguments);
  void run(funkyPars  *pars);
  void print(funkyPars *pars);
  void clean(funkyPars *pars);
  void printArg(FILE *argFile);

  size_t **alleleCounts; //[ind][125]; 
  size_t **alleleCountsChr; //[ind][125]; 
};

void ansErr::printArg(FILE *argFile){
  fprintf(argFile,"--------------\n%s:\n",__FILE__);
  fprintf(argFile,"\t-doAnsError\t%d\n",doAnsError);
  fprintf(argFile,"\t1: EM v1\n");
  fprintf(argFile,"\t2: EM v2\n");
  fprintf(argFile,"\t-sample\t\t%d\tSample only 1 read\n",sample);
  fprintf(argFile,"\n");
}

void ansErr::getOptions(argStruct *arguments){

  //from command line
  doAnsError=angsd::getArg("-doAnsError",doAnsError,arguments);
  sample=angsd::getArg("-sample",sample,arguments);
  nInd=arguments->nInd;
  
  //inputtype 0) soap 1) samglf 2) samglfClean 3) tglf 4)sim type 5) beagle
  if(doAnsError){
    if(arguments->inputtype!=0&&arguments->inputtype!=7){
      fprintf(stderr,"Error: bam or soap input needed for -doAnsError \n");
      exit(0);
    }
  }

}

ansErr::ansErr(const char *outfiles,argStruct *arguments,int inputtype){
  doAnsError=0;
  sample=1;
  currentChr=-1;
  if(arguments->argc==2){
    if(!strcmp(arguments->argv[1],"-doAnsError")){
      printArg(stdout);
      exit(0);
    }else
      return;
  }
  
  getOptions(arguments);
  printArg(arguments->argumentFile);

  if(doAnsError==0)
    return;

  //make output files
  const char* postfix;
  postfix=".ansError";
  outfile = openFile(outfiles,postfix);
  const char* postfix2;
  postfix2=".ansErrorChr";
  outfile2 = openFile(outfiles,postfix2);

  //allocate allele counts
  alleleCounts = new size_t *[nInd];
  for(int i=0;i<nInd;i++)
    alleleCounts[i] = new size_t [256];
  for(int i=0;i<nInd;i++)
    for(int j=0;j<256;j++)
      alleleCounts[i][j]=0;

  alleleCountsChr = new size_t *[nInd];
  for(int i=0;i<nInd;i++)
    alleleCountsChr[i] = new size_t [256];
  for(int i=0;i<nInd;i++)
    for(int j=0;j<256;j++)
      alleleCountsChr[i][j]=0;
}


ansErr::~ansErr(){

  if(doAnsError==0)
    return;

  if(doAnsError==1){
    for(int i=0;i<nInd;i++){
      for(int j=0;j<125;j++)
	fprintf(outfile,"%lu\t",alleleCounts[i][j]);
      fprintf(outfile,"\n");
    }
  }

  for(int i=0;i<nInd;i++)
    delete[]  alleleCounts[i];
  delete [] alleleCounts; 

  for(int i=0;i<nInd;i++)
    delete[]  alleleCountsChr[i];
  delete [] alleleCountsChr; 

  fclose(outfile);
  fclose(outfile2);
}


void ansErr::clean(funkyPars *pars){

}



void ansErr::print(funkyPars *pars){

  if(doAnsError==0)
    return;

  if(doAnsError==2){
    for(int s=0;s<pars->numSites;s++){
      if(pars->keepSites[s]==0)
	continue;
      fprintf(outfile,"%d\t%c",pars->posi[s]+1,intToRef[pars->anc[s]]);
      fprintf(outfile,"\t%c",intToRef[pars->ref[s]]);

     for(int i=0;i<pars->nInd;i++){
       int allele=4;
       if(pars->chk->nd[s][i].l!=0)
	 refToInt[pars->chk->nd[s][i].seq[0]];
	fprintf(outfile,"\t%c",intToRef[allele]);
      }
      fprintf(outfile,"\n");
    }
  }
  else if(doAnsError==1){


    if(sample==0){//use all reads
      for(int s=0;s<pars->numSites;s++){
	if(pars->keepSites[s]==0)
	  continue;
	for(int i=0;i<pars->nInd;i++){
	  for(int j=0;j<pars->chk->nd[s][i].l;j++){
	      alleleCounts[i][pars->anc[s]*25+pars->ref[s]*5+refToInt[pars->chk->nd[s][i].seq[j]]]++;
	      alleleCountsChr[i][pars->anc[s]*25+pars->ref[s]*5+refToInt[pars->chk->nd[s][i].seq[j]]]++;
	  }
	}
      }
    }
    if(sample==1){//random number read
      for(int s=0;s<pars->numSites;s++){
	if(pars->keepSites[s]==0)
	  continue;
	for(int i=0;i<pars->nInd;i++){
	  if(pars->chk->nd[s][i].l==0)
	    continue;
	  int j = std::rand() % pars->chk->nd[s][i].l;
	  alleleCounts[i][pars->anc[s]*25+pars->ref[s]*5+refToInt[pars->chk->nd[s][i].seq[j]]]++;
	  alleleCountsChr[i][pars->anc[s]*25+pars->ref[s]*5+refToInt[pars->chk->nd[s][i].seq[j]]]++;
	}
      }
    }
    if(sample==2){//take first read
      for(int s=0;s<pars->numSites;s++){
	if(pars->keepSites[s]==0)
	  continue;
	for(int i=0;i<pars->nInd;i++){
	  if(pars->chk->nd[s][i].l!=0){
	    alleleCounts[i][pars->anc[s]*25+pars->ref[s]*5+refToInt[pars->chk->nd[s][i].seq[0]]]++;
	    alleleCountsChr[i][pars->anc[s]*25+pars->ref[s]*5+refToInt[pars->chk->nd[s][i].seq[0]]]++;
	  }
	}
      }
    }
  }
   

  if(doAnsError==1){
    if(currentChr==-1)
      currentChr=pars->refId;
    if(currentChr!=pars->refId){
      fprintf(outfile2,"Chr: \t %s\n",header->name[currentChr]);
      for(int i=0;i<nInd;i++){
	for(int j=0;j<125;j++)
	  fprintf(outfile2,"%lu\t",alleleCountsChr[i][j]);
	fprintf(outfile2,"\n");
      }
      for(int i=0;i<nInd;i++)
	for(int j=0;j<256;j++)
	  alleleCountsChr[i][j]=0;
      currentChr=pars->refId;
    }

  }
}


void ansErr::run(funkyPars *pars){

  if(doAnsError==0)
    return;

}






/*
s<-scan("tmp.ansError")
#pars->anc[s]*25+pars->ref[s]*5+refToInt[pars->chk->nd[s][i].seq[j]


fun<-function(a,p,samp)
  s[25*a+5*p+samp+1]

##perfect vs. ancestry
 sum(fun(1,1,0:3));sum(fun(1,0,0:3))

##sample vs. ancestry
 sum(fun(1,0:3,1));sum(fun(1,0:3,0))

##perfect vs. sample
 sum(fun(0:3,1,1));sum(fun(0:3,0,1))

s<-scan("tmp2.ansError")

fun<-function(a,p,samp)
  s[25*a+5*p+samp+1]

##perfect
 sum(fun(1,1,0:3));sum(fun(1,0,0:3))

##sample
 sum(fun(1,0:3,1));sum(fun(1,0:3,0))


s<-scan("tmp3.ansError")
#pars->anc[s]*25+pars->ref[s]*5+refToInt[pars->chk->nd[s][i].seq[j]


fun<-function(a,p,samp)
  s[25*a+5*p+samp+1]

##perfect vs. ancestry
 sum(fun(1,1,0:3));sum(fun(1,0,0:3))

##sample vs. ancestry
 sum(fun(1,0:3,1));sum(fun(1,0:3,0))

##perfect vs. sample
 sum(fun(0:3,1,1));sum(fun(0:3,0,1))

*/
